"use strict";

exports.__esModule = true;
exports.MenuItems = exports.MenuList = exports.MenuPopover = exports.MenuLink = exports.MenuItem = exports.MenuButton = exports.Menu = void 0;

var _react = _interopRequireWildcard(require("react"));

var _portal = _interopRequireDefault(require("@reach/portal"));

var _rect = _interopRequireWildcard(require("@reach/rect"));

var _componentComponent = _interopRequireDefault(require("@reach/component-component"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _utils = require("@reach/utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var noop = function noop() {};

var id = 0;

var genId = function genId() {
  return "button-" + ++id;
}; // TODO: add the mousedown/drag/mouseup to select of native menus, will
// also help w/ remove the menu button tooltip hide-flash.
// TODO: add type-to-highlight like native menus


var MenuContext = (0, _react.createContext)();

var checkIfAppManagedFocus = function checkIfAppManagedFocus(_ref2) {
  var refs = _ref2.refs,
      state = _ref2.state,
      prevState = _ref2.prevState;

  if (!state.isOpen && prevState.isOpen) {
    return !refs.menu.contains(document.activeElement);
  }

  return false;
};

var manageFocusOnUpdate = function manageFocusOnUpdate(_ref3, appManagedFocus) {
  var refs = _ref3.refs,
      state = _ref3.state,
      prevState = _ref3.prevState;

  if (state.isOpen && !prevState.isOpen) {
    window.__REACH_DISABLE_TOOLTIPS = true;

    if (state.selectionIndex !== -1) {
      // haven't measured the popover yet, give it a frame otherwise
      // we'll scroll to the bottom of the page >.<
      requestAnimationFrame(function () {
        if (refs.items && refs.items[state.selectionIndex]) {
          refs.items[state.selectionIndex].focus();
        }
      });
    } else {
      refs.menu.focus();
    }
  } else if (!state.isOpen && prevState.isOpen) {
    if (!appManagedFocus) {
      refs.button && refs.button.focus();
    } // we want to ignore the immediate focus of a tooltip so it doesn't pop
    // up again when the menu closes, only pops up when focus returns again
    // to the tooltip (like native OS tooltips)


    window.__REACH_DISABLE_TOOLTIPS = false;
  } else if (state.selectionIndex !== prevState.selectionIndex) {
    if (state.selectionIndex === -1) {
      // clear highlight when mousing over non-menu items, but focus the menu
      // so the the keyboard will work after a mouseover
      refs.menu && refs.menu.focus();
    } else if (refs.items && refs.items[state.selectionIndex]) {
      refs.items[state.selectionIndex].focus();
    }
  }
};

var openAtFirstItem = function openAtFirstItem(_) {
  return {
    isOpen: true,
    selectionIndex: 0
  };
};

var close = function close(_) {
  return {
    isOpen: false,
    selectionIndex: -1,
    closingWithClick: false
  };
};

var selectItemAtIndex = function selectItemAtIndex(index) {
  return function (_) {
    return {
      selectionIndex: index
    };
  };
};

var getMenuRefs = function getMenuRefs() {
  return {
    button: null,
    menu: null,
    items: []
  };
};

var getInitialMenuState = function getInitialMenuState() {
  return {
    buttonId: null,
    isOpen: false,
    buttonRect: undefined,
    selectionIndex: -1,
    closingWithClick: false
  };
};

var checkIfStylesIncluded = function checkIfStylesIncluded() {
  return (0, _utils.checkStyles)("menu-button");
}; ////////////////////////////////////////////////////////////////////////////////


var Menu = function Menu(_ref4) {
  var children = _ref4.children;
  return _react["default"].createElement(_componentComponent["default"], {
    getRefs: getMenuRefs,
    getInitialState: getInitialMenuState,
    didMount: checkIfStylesIncluded,
    didUpdate: manageFocusOnUpdate,
    getSnapshotBeforeUpdate: checkIfAppManagedFocus
  }, function (context) {
    return _react["default"].createElement(MenuContext.Provider, {
      value: _extends({}, context)
    }, typeof children === "function" ? children({
      isOpen: context.state.isOpen
    }) : children);
  });
};

exports.Menu = Menu;

if (process.env.NODE_ENV !== "production") {
  Menu.propTypes = {
    children: _propTypes["default"].oneOfType([_propTypes["default"].func, _propTypes["default"].node])
  };
  Menu.displayName = "Menu";
} ////////////////////////////////////////////////////////////////////////////////


var MenuButton = (0, _react.forwardRef)(function MenuButton(_ref5, forwardedRef) {
  var onClick = _ref5.onClick,
      onKeyDown = _ref5.onKeyDown,
      onMouseDown = _ref5.onMouseDown,
      id = _ref5.id,
      props = _objectWithoutPropertiesLoose(_ref5, ["onClick", "onKeyDown", "onMouseDown", "id"]);

  var _useContext = (0, _react.useContext)(MenuContext),
      refs = _useContext.refs,
      state = _useContext.state,
      setState = _useContext.setState;

  var ownRef = (0, _react.useRef)(null);
  (0, _rect.useRect)(ownRef, state.isOpen, function (buttonRect) {
    return setState({
      buttonRect: buttonRect
    });
  });
  (0, _react.useEffect)(function () {
    return setState({
      buttonId: id != null ? id : genId()
    });
  }, [] // eslint-disable-line react-hooks/exhaustive-deps
  );
  return _react["default"].createElement("button", _extends({
    id: state.buttonId,
    "aria-haspopup": "menu",
    "aria-expanded": state.isOpen,
    "data-reach-menu-button": true,
    type: "button",
    ref: function ref(node) {
      (0, _utils.assignRef)(forwardedRef, node);
      (0, _utils.assignRef)(ownRef, node);
      refs.button = node;
    },
    onMouseDown: (0, _utils.wrapEvent)(onMouseDown, function () {
      if (state.isOpen) {
        setState({
          closingWithClick: true
        });
      }
    }),
    onClick: (0, _utils.wrapEvent)(onClick, function () {
      if (state.isOpen) {
        setState(close);
      } else {
        setState(openAtFirstItem);
      }
    }),
    onKeyDown: (0, _utils.wrapEvent)(onKeyDown, function (event) {
      if (event.key === "ArrowDown") {
        event.preventDefault(); // prevent scroll

        setState(openAtFirstItem);
      } else if (event.key === "ArrowUp") {
        event.preventDefault(); // prevent scroll

        setState(openAtFirstItem);
      }
    })
  }, props));
});
exports.MenuButton = MenuButton;

if (process.env.NODE_ENV !== "production") {
  MenuButton.propTypes = {
    onClick: _propTypes["default"].func,
    onKeyDown: _propTypes["default"].func,
    children: _propTypes["default"].node
  };
  MenuButton.displayName = "MenuButton";
} ////////////////////////////////////////////////////////////////////////////////


var MenuItem = (0, _react.forwardRef)(function MenuItem(_ref6, forwardedRef) {
  var onClick = _ref6.onClick,
      onKeyDown = _ref6.onKeyDown,
      onMouseLeave = _ref6.onMouseLeave,
      onMouseMove = _ref6.onMouseMove,
      onSelect = _ref6.onSelect,
      _ref6$role = _ref6.role,
      role = _ref6$role === void 0 ? "menuitem" : _ref6$role,
      index = _ref6._index,
      _ref6$_ref = _ref6._ref,
      _ref = _ref6$_ref === void 0 ? null : _ref6$_ref,
      rest = _objectWithoutPropertiesLoose(_ref6, ["onClick", "onKeyDown", "onMouseLeave", "onMouseMove", "onSelect", "role", "_index", "_ref"]);

  var _useContext2 = (0, _react.useContext)(MenuContext),
      state = _useContext2.state,
      setState = _useContext2.setState;

  var ownRef = (0, _react.useRef)(null);
  var ref = (0, _utils.useForkedRef)(_ref, forwardedRef, ownRef);
  var isSelected = index === state.selectionIndex;

  var select = function select() {
    onSelect();
    setState(close);
  };

  return _react["default"].createElement("div", _extends({}, rest, {
    ref: ref,
    "data-reach-menu-item": role === "menuitem" ? true : undefined,
    role: role,
    tabIndex: -1,
    "data-selected": role === "menuitem" && isSelected ? true : undefined,
    onClick: (0, _utils.wrapEvent)(onClick, function (event) {
      select();
    }),
    onKeyDown: (0, _utils.wrapEvent)(onKeyDown, function (event) {
      if (event.key === "Enter" || event.key === " ") {
        // prevent the button from being "clicked" by
        // this "Enter" keydown
        event.preventDefault();
        select();
      }
    }),
    onMouseMove: (0, _utils.wrapEvent)(onMouseMove, function (event) {
      if (!isSelected) {
        setState(selectItemAtIndex(index));
      }
    }),
    onMouseLeave: (0, _utils.wrapEvent)(onMouseLeave, function (event) {
      // clear out selection when mouse over a non-menu item child
      setState({
        selectionIndex: -1
      });
    })
  }));
});
exports.MenuItem = MenuItem;

if (process.env.NODE_ENV !== "production") {
  MenuItem.propTypes = {
    onSelect: _propTypes["default"].func.isRequired,
    onClick: _propTypes["default"].func,
    role: _propTypes["default"].string,
    state: _propTypes["default"].object,
    setState: _propTypes["default"].func,
    onKeyDown: _propTypes["default"].func,
    onMouseMove: _propTypes["default"].func,
    _ref: _propTypes["default"].func,
    _index: _propTypes["default"].number
  };
  MenuItem.displayName = "MenuItem";
} ////////////////////////////////////////////////////////////////////////////////


var MenuLink = (0, _react.forwardRef)(function MenuLink(_ref7, forwardedRef) {
  var _ref7$as = _ref7.as,
      AsComp = _ref7$as === void 0 ? "a" : _ref7$as,
      Comp = _ref7.component,
      onClick = _ref7.onClick,
      onKeyDown = _ref7.onKeyDown,
      role = _ref7.role,
      index = _ref7._index,
      _ref7$_ref = _ref7._ref,
      _ref = _ref7$_ref === void 0 ? null : _ref7$_ref,
      props = _objectWithoutPropertiesLoose(_ref7, ["as", "component", "onClick", "onKeyDown", "role", "_index", "_ref"]);

  var _useContext3 = (0, _react.useContext)(MenuContext),
      state = _useContext3.state,
      setState = _useContext3.setState;

  var Link = Comp || AsComp;
  var ownRef = (0, _react.useRef)(null);
  var ref = (0, _utils.useForkedRef)(_ref, forwardedRef, ownRef);

  if (Comp) {
    console.warn("[@reach/menu-button]: Please use the `as` prop instead of `component`.");
  }

  return _react["default"].createElement(MenuItem, {
    role: "none",
    onSelect: noop,
    _index: index,
    _ref: noop
  }, _react["default"].createElement(Link, _extends({
    role: "menuitem",
    "data-reach-menu-item": true,
    tabIndex: -1,
    "data-selected": index === state.selectionIndex ? true : undefined,
    onClick: (0, _utils.wrapEvent)(onClick, function (event) {
      setState(close);
    }),
    onKeyDown: (0, _utils.wrapEvent)(onKeyDown, function (event) {
      if (event.key === "Enter") {
        // prevent MenuItem's preventDefault from firing,
        // allowing this link to work w/ the keyboard
        event.stopPropagation();
      }
    }),
    ref: ref
  }, props)));
});
exports.MenuLink = MenuLink;

if (process.env.NODE_ENV !== "production") {
  MenuLink.propTypes = {
    as: _propTypes["default"].any,
    component: _propTypes["default"].any,
    onClick: _propTypes["default"].func,
    onKeyDown: _propTypes["default"].func,
    _index: _propTypes["default"].number,
    _ref: _propTypes["default"].func
  };
  MenuLink.displayName = "MenuLink";
} ////////////////////////////////////////////////////////////////////////////////


var MenuPopover = (0, _react.forwardRef)(function MenuPopover(_ref8, forwardedRef) {
  var children = _ref8.children,
      style = _ref8.style,
      props = _objectWithoutPropertiesLoose(_ref8, ["children", "style"]);

  var _useContext4 = (0, _react.useContext)(MenuContext),
      state = _useContext4.state;

  return state.isOpen && _react["default"].createElement(_portal["default"], null, _react["default"].createElement(_rect["default"], null, function (_ref9) {
    var menuRect = _ref9.rect,
        _ref10 = _ref9.ref;
    return _react["default"].createElement("div", _extends({
      "data-reach-menu-popover": true,
      "data-reach-menu": true // deprecate for naming consistency?
      ,
      ref: function ref(node) {
        (0, _utils.assignRef)(_ref10, node);
        (0, _utils.assignRef)(forwardedRef, node);
      }
    }, props, {
      style: _extends({}, style, {}, getStyles(state.buttonRect, menuRect))
    }), children);
  }));
});
exports.MenuPopover = MenuPopover;

if (process.env.NODE_ENV !== "production") {
  MenuPopover.propTypes = {
    children: _propTypes["default"].node
  };
  MenuPopover.displayName = "MenuPopover";
} ////////////////////////////////////////////////////////////////////////////////


var MenuList = (0, _react.forwardRef)(function MenuList(props, forwardedRef) {
  var ownRef = (0, _react.useRef)(null);
  var ref = (0, _utils.useForkedRef)(ownRef, forwardedRef);
  return _react["default"].createElement(MenuPopover, null, _react["default"].createElement(MenuItems, _extends({}, props, {
    "data-reach-menu-list": "",
    ref: ref
  })));
});
exports.MenuList = MenuList;

if (process.env.NODE_ENV !== "production") {
  MenuList.propTypes = {
    children: _propTypes["default"].node.isRequired
  };
  MenuList.displayName = "MenuList";
} ////////////////////////////////////////////////////////////////////////////////


var focusableChildrenTypes = [MenuItem, MenuLink];

var isFocusableChildType = function isFocusableChildType(child) {
  return focusableChildrenTypes.includes(child.type);
};

var getFocusableMenuChildren = function getFocusableMenuChildren(childrenArray) {
  var focusable = childrenArray.filter(function (child) {
    return isFocusableChildType(child);
  });
  return focusable;
}; ////////////////////////////////////////////////////////////////////////////////


var MenuItems = (0, _react.forwardRef)(function MenuItems(_ref11, _ref12) {
  var children = _ref11.children,
      onKeyDown = _ref11.onKeyDown,
      onBlur = _ref11.onBlur,
      rest = _objectWithoutPropertiesLoose(_ref11, ["children", "onKeyDown", "onBlur"]);

  var _useContext5 = (0, _react.useContext)(MenuContext),
      state = _useContext5.state,
      setState = _useContext5.setState,
      refs = _useContext5.refs;

  var clones = _react.Children.toArray(children).filter(Boolean);

  var focusableChildren = getFocusableMenuChildren(clones);
  return _react["default"].createElement("div", _extends({
    "data-reach-menu-items": true
  }, rest, {
    role: "menu",
    "aria-labelledby": state.buttonId,
    tabIndex: -1,
    ref: function ref(node) {
      refs.menu = node;
      (0, _utils.assignRef)(_ref12, node);
    },
    onBlur: function onBlur(event) {
      if (!state.closingWithClick && !refs.menu.contains(event.relatedTarget || document.activeElement)) {
        setState(close);
      }
    },
    onKeyDown: (0, _utils.wrapEvent)(onKeyDown, function (event) {
      if (event.key === "Escape") {
        setState(close);
      } else if (event.key === "ArrowDown") {
        event.preventDefault(); // prevent window scroll

        var nextIndex = state.selectionIndex + 1;

        if (nextIndex !== focusableChildren.length) {
          setState({
            selectionIndex: nextIndex
          });
        }
      } else if (event.key === "ArrowUp") {
        event.preventDefault(); // prevent window scroll

        var _nextIndex = state.selectionIndex - 1;

        if (_nextIndex !== -1) {
          setState({
            selectionIndex: _nextIndex
          });
        }
      } else if (event.key === "Tab") {
        event.preventDefault(); // prevent leaving
      }
    })
  }), clones.map(function (child) {
    if (isFocusableChildType(child)) {
      var focusIndex = focusableChildren.indexOf(child);
      return (0, _react.cloneElement)(child, {
        _index: focusIndex,
        _ref: function _ref(node) {
          return refs.items[focusIndex] = node;
        }
      });
    }

    return child;
  }));
});
exports.MenuItems = MenuItems;

if (process.env.NODE_ENV !== "production") {
  MenuItems.propTypes = {
    refs: _propTypes["default"].object,
    state: _propTypes["default"].object,
    setState: _propTypes["default"].func,
    children: _propTypes["default"].node,
    onKeyDown: _propTypes["default"].func,
    onBlur: _propTypes["default"].func
  };
  MenuItems.displayName = "MenuItems";
} ////////////////////////////////////////////////////////////////////////////////


var getStyles = function getStyles(buttonRect, menuRect) {
  var haventMeasuredButtonYet = !buttonRect;

  if (haventMeasuredButtonYet) {
    return {
      opacity: 0
    };
  }

  var haventMeasuredMenuYet = !menuRect;
  var styles = {
    left: buttonRect.left + window.pageXOffset + "px",
    top: buttonRect.top + buttonRect.height + window.pageYOffset + "px"
  };

  if (haventMeasuredMenuYet) {
    return _extends({}, styles, {
      opacity: 0
    });
  }

  if (buttonRect.width < 500) {
    styles.minWidth = buttonRect.width;
  }

  var collisions = {
    top: buttonRect.top - menuRect.height < 0,
    right: window.innerWidth < buttonRect.left + menuRect.width,
    bottom: window.innerHeight < buttonRect.top + menuRect.height,
    left: buttonRect.left + buttonRect.width - menuRect.width < 0
  };
  var directionRight = collisions.right && !collisions.left;
  var directionUp = collisions.bottom && !collisions.top;
  return _extends({}, styles, {
    left: directionRight ? buttonRect.right - menuRect.width + window.pageXOffset + "px" : buttonRect.left + window.pageXOffset + "px",
    top: directionUp ? buttonRect.top - menuRect.height + window.pageYOffset + "px" : buttonRect.top + buttonRect.height + window.pageYOffset + "px"
  });
};